/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2006-2008 - INRIA - 
* Copyright (C) 2010 - DIGITEO - Allan CORNET
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*--------------------------------------------------------------------------*/
#include <string.h>

// From Distfun:
#include "gwsupport.h"
#include "gw_distfuncdf.h"
#include "cdflib.h"
#include "gw_distfuncdf_support.h"

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"
#include "Scierror.h"
#include "localization.h"

/*--------------------------------------------------------------------------*/
/*

Negative BiNomial Inverse CDF

Calling sequences :

X=distfun_invnbn(Xn,Pr,Ompr,P,Q)

*/
/*--------------------------------------------------------------------------*/
int sci_distfun_invnbn(char* fname,unsigned long l)
{
	int readFlag;

	int rowsXn = 0, colsXn = 0;
	int rowsPr = 0, colsPr = 0;
	int rowsOm = 0, colsOm = 0;
	int rowsP = 0, colsP = 0;
	int rowsQ = 0, colsQ = 0;
	int rowsX = 0, colsX = 0;

	double * lrXn = NULL;
	double * lrPr = NULL;
	double * lrOm = NULL;
	double * lrP = NULL;
	double * lrQ = NULL;
	double * lrX = NULL;

	int which = 2;
	double bound = 0;
	int status = 0;

	int i;

	CheckInputArgument(pvApiCtx,5,5);
	CheckOutputArgument(pvApiCtx,1,1);

	// Arg #1 : Xn
	readFlag = distfun_GetSizedRealMatrixOfDoublesGreaterOrEqualThanZero( fname, 1, -1, -1, &lrXn, &rowsXn , &colsXn );
	if(readFlag==DISTFUNCDFGW_ERROR)
	{
		return 0;
	}
	// Arg #2 : Pr, arg #3 : Ompr
	readFlag = distfun_GetMatricesPQ( fname, 2,  rowsXn, colsXn,
		&lrPr, &rowsPr , &colsPr, &lrOm, &rowsOm , &colsOm);
	if(readFlag==DISTFUNCDFGW_ERROR)
	{
		return 0;
	}
	// Arg #4 : P, arg #5 : Q
	readFlag = distfun_GetMatricesPQ( fname, 4,  rowsXn, colsXn,
		&lrP, &rowsP , &colsP, &lrQ, &rowsQ , &colsQ);
	if(readFlag==DISTFUNCDFGW_ERROR)
	{
		return 0;
	}
	// Create LHS : X
	rowsX = rowsXn;
	colsX = colsXn;
	iAllocMatrixOfDouble ( Rhs + 1 , rowsX , colsX , &lrX );
	LhsVar(1) = Rhs+1;
	// Fill X
	status = CDFLIB_OK;
	for ( i=0 ; i < rowsX*colsX; i++)
	{
		status=cdflib_cdfnbn(which, lrP+i, lrQ+i, lrX+i, lrXn[i], lrPr[i], lrOm[i]);

		if (status != CDFLIB_OK)
		{
			distfun_defaultCDFError( fname, i );
			break;
		}
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
