// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <math.h>
#include <stdio.h>

#include "cdflib.h" 
#include "cdflib_private.h" 

// Beta PDF
int cdflib_betapdf(double x, double a, double b, double *y)
{
	double lny;
	int status;
	double z;
	double t1, t2, t3;
	double infinite=cdflib_infinite();
	//
	// Check arguments
	status=cdflib_betCheckX("cdflib_betapdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	status=cdflib_betCheckParams("cdflib_betapdf",a,b);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	//
	// Compute t1=(a-1)*log(x)
	//
	if (x==0)
	{
		if (a<1)
		{
			*y=infinite;
			return (CDFLIB_OK);
		}
		else if (a>1)
		{
			*y=0;
			return (CDFLIB_OK);
		}
		else
		{
			t1=0;
		}
	}
	else
	{
		t1=(a-1)*log(x);
	}
	//
	// Compute t2=(b-1)*log(1-x)
	//
	if (x==1)
	{
		if (b<1)
		{
			*y=infinite;
			return (CDFLIB_OK);
		}
		else if (b>1)
		{
			*y=0;
			return (CDFLIB_OK);
		}
		else
		{
			t2=0;
		}
	}
	else
	{
		z=-x;
		t2=(b-1)*cdflib_log1p(z);
	}
	if (a>2 && b>2)
	{
		// Evaluate (a+b-1)*binompdf(a-1,a+b-2,x)
		lny = log(a+b-1) + cdflib_binopdfraw(a-1,a+b-2,x,1-x,1);
	}
	else
	{
		// Evaluate (a-1)*log(x) + (b-1)*log(1-x) - log(beta(a,b))
		t3=cdflib_betaln(a, b);
		lny = t1+t2-t3;
	}
	*y = exp ( lny );
	return ( CDFLIB_OK );
}

