/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfchn.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/


#include "cdflib.h"
#include "cdflib_private.h" 


/* Table of constant values */

static double c_b15 = 0.;
static double c_b17 = .5;
static double c_b19 = 5.;
static double c_b20 = 1e-50;
static double c_b40 = 1e4;

/* Subroutine */ int cdflib_cdfchn(int which, double *p, double *q, 
	double *x, double df, double pnonc)
{
	int status;

	static double fx;
	static long int qhi;
	static double cum, ccum;
	static long int qleft;

	double large = cdflib_doubleLarge();
	double rtol = cdflib_doubleEps();

	/* ********************************************************************** */

	/*               Cumulative Distribution Function */
	/*               Non-central Chi-Square */


	/*                              Function */


	/*     Calculates any one parameter of the non-central chi-square */
	/*     distribution given values for the others. */


	/*                              Arguments */


	/*     WHICH --> Integer indicating which of the next three argument */
	/*               values is to be calculated from the others. */
	/*               Input range: 1,2 */
	/*               iwhich = 1 : Calculate P and Q from X and DF */
	/*               iwhich = 2 : Calculate X from P,DF and PNONC */
	

	/*     P <--> The integral from 0 to X of the non-central chi-square */
	/*            distribution. */
	/*            Input range: [0, 1-1E-16). */
	

	/*     Q <--> 1-P. */
	/*            Input range: (0, 1]. */
	/*            P + Q = 1.0. */
	

	/*     X <--> Upper limit of integration of the non-central */
	/*            chi-square distribution. */
	/*            Input range: [0, +infinity). */
	/*            Search range: [0,large] */
	

	/*     DF <--> Degrees of freedom of the non-central */
	/*             chi-square distribution. */
	/*             Input range: (0, +infinity). */
	/*             Search range: [ atol, large] */


	/*     PNONC <--> Non-centrality parameter of the non-central */
	/*                chi-square distribution. */
	/*                Input range: [0, +infinity). */
	/*                Search range: [0,1E4] */


	/*     STATUS <-- CDFLIB_OK if calculation completed correctly */
	/*                CDFLIB_ERROR if not */

	/*                              Method */


	/*     Formula  26.4.25   of   Abramowitz   and   Stegun,  Handbook  of */
	/*     Mathematical  Functions (1966) is used to compute the cumulative */
	/*     distribution function. */

	/*     Computation of other parameters involve a seach for a value that */
	/*     produces  the desired  value  of P.   The search relies  on  the */
	/*     monotinicity of P with the other parameter. */


	/*                            WARNING */

	/*     The computation time  required for this  routine is proportional */
	/*     to the noncentrality  parameter  (PNONC).  Very large  values of */
	/*     this parameter can consume immense  computer resources.  This is */
	/*     why the search range is bounded by 10,000. */

	/* ********************************************************************** */

	/*     Check arguments */

	status=cdflib_checkrangeint("cdflib_cdfchn", which, "which", 1, 2);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/* P, Q, P + Q */
	if (which==2)
	{
		status=cdflib_checkpq("cdflib_cdfchn", *p, "p", *q, "p");
		if (status!=CDFLIB_OK)
		{
			return status;
		}
	}

	// Check X
	if (which==1)
	{
		status=cdflib_chnCheckX("cdflib_cdfchn",*x);
		if (status!=CDFLIB_OK)
		{
			return status;
		}
	}

	// Check DF, PNONC
	status = cdflib_chnCheckParams("cdflib_cdfchn",df, pnonc);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     Calculate ANSWERS */

	if (which==1) 
	{
		/*     Calculating P and Q */
		if (cdflib_isnan(*x) || cdflib_isnan(df) || cdflib_isnan(pnonc))
		{
			*p = *x+df+pnonc;
			*q = *x+df+pnonc;
			status = CDFLIB_OK;
			return status;
		}
		cdflib_cumchn(*x, df, pnonc, p, q);
		status = CDFLIB_OK;
	} 
	else if (which==2) 
	{
		/*     Calculating X */
		if (cdflib_isnan(*p) || cdflib_isnan(*q) || cdflib_isnan(df) || cdflib_isnan(pnonc))
		{
			*x = *p+*q+df+pnonc;
			status = CDFLIB_OK;
			return status;
		}
		*x = 5.;
		cdflib_dstinv(&c_b15, &large, &c_b17, &c_b17, &c_b19, &c_b20, &rtol);
		status = 0;
		cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
		while (status == 1)
		{
			cdflib_cumchn(*x, df, pnonc, &cum, &ccum);
			fx = cdflib_computefx(*p, *q, cum, ccum);
			cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
		}
		if (status == 0) 
		{
			status = CDFLIB_OK;
		}
		else
		{
			if (qleft) 
			{
				cdflib_unabletoinvert("cdflib_cdfchn", 0., "x");
				status = CDFLIB_ERROR;
			}
			else
			{
				cdflib_unabletoinvert("cdflib_cdfchn", large, "x");
				status = CDFLIB_ERROR;
			}
		}
	}
	return status;
}

int cdflib_chnCheckParams(char * fname, double df, double pnonc)
{
	int status;
	/*     DF > 0 */
	status = cdflib_checkgreaterthan(fname, df, "df", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}


	/*     PNONC >= 0 */
	status = cdflib_checkgreqthan(fname, pnonc, "pnonc", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

int cdflib_chnCheckX(char * fname, double x)
{
	/*     X >=0 */
	int status;
	status=cdflib_checkgreqthan(fname, x, "x", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
