/* cdft.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

/* ********************************************************************** */

// T Distribution

/*                              Method */

/*     Formula  26.5.27  of   Abramowitz   and  Stegun,   Handbook   of */
/*     Mathematical Functions  (1966) is used to reduce the computation */
/*     of the cumulative distribution function to that of an incomplete */
/*     beta. */

/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */

/* ********************************************************************** */

int cdflib_tCheckX(char * fname, double x)
{
	// Nothing to do.
	return CDFLIB_OK;
}
int cdflib_tCheckParams(char * fname, double v)
{
	int status;
	/*     V > 0 */
	status=cdflib_checkgreaterthan(fname, v, "v", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_tcdf(double x, double v, int lowertail, double *p)
{
	int status;
	double infinite = cdflib_infinite();
	double q;
	status=cdflib_tCheckX("cdflib_tcdf", x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	// Check V
	status=cdflib_tCheckParams("cdflib_tcdf", v);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	if (cdflib_isnan(x) || cdflib_isnan(v))
	{
		*p = x+v;
		status = CDFLIB_OK;
		return status;
	}
	if (v==infinite)
	{
		cdflib_cumnor(x, p, &q);
		if (lowertail==CDFLIB_UPPERTAIL)
		{
			*p=q;
		}
		status = CDFLIB_OK;
		return status;
	}
	cdflib_cumt(x, v, p, &q);
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	status = CDFLIB_OK;
	return status;
}
int cdflib_tinv(double p, double v, int lowertail, double *x)
{
	int status;
	double q;
	/* Local variables */
	static double fx, pq;
	static long int qhi;
	static double cum, ccum;
	static long int qleft;
	static long int qporq;

	double rtol = cdflib_doubleEps();
	double zero = cdflib_doubleTiny();
	double infinite = cdflib_infinite();
	double mu=0.;
	double sigma=1.;

	/* Table of constant values */

	static int c__1 = 1;
	static double c_b20 = -1e150;
	static double c_b21 = 1e150;
	static double c_b22 = .5;
	static double c_b24 = 5.;
	static double c_b25 = 1e-50;

	// P in [0,1]
	status=cdflib_checkp("cdflib_tinv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check V
	status=cdflib_tCheckParams("cdflib_tinv", v);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}
	if ( p == 0. ) 
	{
		*x = -infinite;
		status = CDFLIB_OK;
		return status;
	}
	if ( q == 0. ) 
	{
		*x = infinite;
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(p) || cdflib_isnan(v))
	{
		*x = p+v;
		status = CDFLIB_OK;
		return status;
	}
	if (v==infinite)
	{
		if (p<q)
		{
			status = cdflib_norminv(p, mu, sigma, CDFLIB_LOWERTAIL, x);
		}
		else
		{
			status = cdflib_norminv(q, mu, sigma, CDFLIB_UPPERTAIL, x);
		}
		return status;
	}
	/*     .. Get initial approximation for X */
	*x = cdflib_dt1(p, q, v);
	cdflib_dstinv(&c_b20, &c_b21, &c_b22, &c_b22, &c_b24, &c_b25, &rtol);
	status = 0;
	cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
	while (status == 1)
	{
		cdflib_cumt(*x, v, &cum, &ccum);
		fx = cdflib_computefx(p, q, cum, ccum);
		cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
	}
	if (status == 0) 
	{
		status = CDFLIB_OK;
	}
	else
	{
		if (qleft) 
		{
			cdflib_unabletoinvert("cdflib_tinv", c_b20, "x");
			status = CDFLIB_ERROR;
		}
		else
		{
			cdflib_unabletoinvert("cdflib_tinv", c_b21, "x");
			status = CDFLIB_ERROR;
		}
	} 
	return status;
}
