/* cumf.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
  */

#include "cdflib.h"
#include "cdflib_private.h" 

/* Subroutine */ int cdflib_cumf(double x, double dfn, double dfd, 
	double *cum, double *ccum)
{
	/* System generated locals */
	double d__1, d__2;

	/* Local variables */
	static double xx, yy;
	static int ierr;
	static double prod, dsum;
	double infinite = cdflib_infinite();

	/* ********************************************************************** */

	/*     SUBROUTINE CUMF(X,DFN,DFD,CUM,CCUM) */
	/*                    CUMulative F distribution */


	/*                              Function */


	/*     Computes  the  integral from  0  to  X of  the f-density  with DFN */
	/*     and DFD degrees of freedom. */


	/*                              Arguments */


	/*     X --> Upper limit of integration of the f-density. */


	/*     DFN --> Degrees of freedom of the numerator sum of squares. */

	/*     DFD --> Degrees of freedom of the denominator sum of squares. */

	/*     CUM <-- Cumulative f distribution. */

	/*     CCUM <-- Compliment of Cumulative f distribution. */


	/*                              Method */


	/*     Formula  26.5.28 of  Abramowitz and   Stegun   is  used to  reduce */
	/*     the cumulative F to a cumulative beta distribution. */


	/*                              Note */


	/*     If X is less than or equal to 0, 0 is returned. */

	/* ********************************************************************** */

	if (x <= 0.) 
	{
		*cum = 0.;
		*ccum = 1.;
		return 0;
	}
	if ( (dfd==infinite) & (dfn==infinite) )
	{
		if (x<1)
		{
			*cum = 0.;
			*ccum = 1.;
			return 0;
		} 
		else if (x==1)
		{
			*cum = 0.5;
			*ccum = 0.5;
			return 0;
		}
		else
		{
			// f>1
			*cum = 1.;
			*ccum = 0.;
			return 0;
		}
	}

	prod = dfn * x;

	/*     XX is such that the incomplete beta with parameters */
	/*     DFD/2 and DFN/2 evaluated at XX is 1 - CUM or CCUM */

	/*     YY is 1 - XX */

	/*     Calculate the smaller of XX and YY accurately */

	dsum = dfd + prod;
	xx = dfd / dsum;
	if (xx > .5) 
	{
		yy = prod / dsum;
		xx = 1. - yy;
	} 
	else 
	{
		yy = 1. - xx;
	}
	d__1 = dfd * .5;
	d__2 = dfn * .5;
	cdflib_bratio(d__1, d__2, xx, yy, ccum, cum, &ierr);
	return 0;
}

