// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <math.h>
#include <stdio.h>

#include "cdflib.h" 
#include "cdflib_private.h" 

int cdflib_fpdf(double x,double v1,double v2, double * y)
{
	double f;
	double p;
	double q;
	int status;
	double inf=cdflib_infinite();	
	//
	// Check arguments.
	status=cdflib_fCheckX("cdflib_fpdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// 
	status=cdflib_fCheckParams("cdflib_fpdf",v1, v2);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	//
	// Perform...
	if ((v1==inf) & (v2==inf))
	{
		if (x==1)
		{
			*y = inf;
			return CDFLIB_OK;
		} 
		else
		{
			*y = 0;
			return CDFLIB_OK;
		}
	}
	if (x==0)
	{
		if (v1<2)
		{
			*y = inf;
			return CDFLIB_OK;
		}
		else if (v1==2)
		{
			*y = 1.;
			return CDFLIB_OK;
		}
		else
		{
			// v1>2
			*y = 0.;
			return CDFLIB_OK;
		}
	}
	
    if (v1 > 1e14) 
	{
        // includes +Inf: code below is inaccurate there
        status = cdflib_gammapdf(1 / x, v2/2, 2 /v2, y);
		if (status==CDFLIB_ERROR)
		{	
			return CDFLIB_ERROR;
		}
        *y = *y /(x*x);
		return CDFLIB_OK;
    }
    // General case
    f = 1 /(v2+x*v1);
    q = v2*f;
    p = x*v1*f;
    //
    if (v1>=2)
	{
        f = v1*q/2;
        *y = cdflib_binopdfraw((v1-2)/2, (v1+v2-2)/2, p, q,0);
    }
    else
	{
        f = v1*v1*q / (2*p*(v1+v2));
        *y = cdflib_binopdfraw(v1/2, (v1+v2)/2, p, q,0);
    }
    *y = f*(*y);
    return CDFLIB_OK;
}

