// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <math.h>
#include <stdio.h>

#include "cdflib.h" 
#include "cdflib_private.h" 

int cdflib_gammapdf( double x , double a , double b, double * y )
{
	double logy;
	int status;
	double infinite = cdflib_infinite();
	//
	// Check arguments
	//
	status=cdflib_gamCheckX("cdflib_gammapdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	status=cdflib_gamCheckParams("cdflib_gammapdf",a, b);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
    //
	if (x==infinite)
	{
		*y=0.;
		return CDFLIB_OK;
	}
    // x<>0
    if (x!=0)
	{
        logy = (a-1)*log(x) - x/b - a*log(b) - cdflib_gamln(a);
        *y=exp(logy);
	    return ( CDFLIB_OK );
	}
    //
    // x=0, a=1
    if ((x==0) & (a==1))
	{
        *y=1/b;
	    return ( CDFLIB_OK );
	}
    //
    // x=0, a<1
    if ((x==0)&(a<1))
	{
        *y=cdflib_infinite();
	    return ( CDFLIB_OK );
	}
	*y = 0.;
	return ( CDFLIB_OK );
}
