// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <math.h>
#include <stdio.h>

#include "cdflib.h" 
#include "cdflib_private.h" 

// Computes the Poisson PDF
int cdflib_poisspdf(double x, double lambda, double *y)
{
	double t;
	int status;
	//
	// Check arguments
	//
	status=cdflib_poissCheckX("cdflib_poisspdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	status=cdflib_poissCheckParams("cdflib_poisspdf",lambda);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	//
	// Proceed...
	if (x==0)
	{
		*y=exp(-lambda);
	}
	else
	{
		t=cdflib_rcomp(x, lambda);
		*y=t/x;
	}
	return ( CDFLIB_OK );
}

int cdflib_poissCheckX(char * fname, double x)
{
	int status;

	// X has integer value
	status=cdflib_checkIntValue(fname, x, "x");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	/*     X >= 0 */
	status=cdflib_checkgreqthan(fname, x, "x", 0);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

int cdflib_poissCheckParams(char * fname, double lambda)
{
	int status;
	/*     LAMBDA > 0 */
	status=cdflib_checkgreaterthan(fname, lambda, "lambda", 0);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
