// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <math.h>

#include "cdflib.h" 
#include "cdflib_private.h" 

// T PDF
int cdflib_tpdf(double x, double v, double *y)
{
	double lny;
	int status;
	double t;
	double half=0.5;
	double u;
	double halfv;
	double s;
	double inf=cdflib_infinite();
	//
	// Check arguments
	status=cdflib_tCheckX("cdflib_tpdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	status=cdflib_tCheckParams("cdflib_tpdf",v);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	//
	if (v==inf)
	{
		status=cdflib_normpdf(x, 0., 1., y);
		return status;
	}
	// Computes t = log(1+x^2/v)
	s = x*x/v;
	t = cdflib_log1p(s);
	// Computes u = log(B(v/2,1/2))
	halfv=v/2;
	u = cdflib_betaln(halfv, half);
	lny = - 0.5*(v+1)*t-0.5*log(v)-u;
	*y = exp ( lny );
	return ( CDFLIB_OK );
}

