// Copyright (C) 2012 - Michael Baudin
#include <math.h>
#include "genrand.h"
#include "genrand_private.h"


/*
This source code was taken in the project "freemat" (BSD license)
This source code was modified by Gaüzère Sabine according to the 
modifications done by JJV
*/
double genrand_genbet(double a,double b)
{
	// expmax is so that exp(expmax)=huge
	double expmax = 709.78271289338397309621;
	double tiny = genrand_doubleTiny();
	double huge = genrand_doubleHuge();
	static double olda = -1;
	static double oldb = -1;

	static double aa,alpha,bb,beta,delta,gamma,k1,k2,r,s,t,u1,u2,v,w,y,z;
	double R;
	static int qsame;
	double c1=1.3888900168240070343E-2;
	double c2=4.1666701436042785645E-2;
	double c3=0.77777802944183349609;
	double c4=1.3862943649291992188;

	qsame = olda == a && oldb == b;
	if(!qsame) {
		olda = a;
		oldb = b;
	}
	if(!(min(a,b) > 1.0)) {
		goto S100;
	}
	/*
	Alborithm BB
	Initialize
	*/
	if(qsame) {
		goto S30;
	}
	aa = min(a,b);
	bb = max(a,b);
	alpha = aa+bb;
	beta = sqrt((alpha-2.0)/(2.0*aa*bb-alpha));
	gamma = aa+1.0/beta;
S30:
S40:
	u1 = genrand_randgenerate();
	/*
	Step 1
	*/
	u2 = genrand_randgenerate();
	v = beta*log(u1/(1.0-u1));
	if(v > expmax) {
		goto S55;
	}
	w= exp(v);
	if(w > (huge/aa)) {
		goto S55;
	}
	w *= aa;
	goto S60;

S55:
	w = huge;
S60:
	z = pow(u1,2.0f)*u2;
	r = gamma*v-c4;
	s = aa+r-w;
	/*
	Step 2
	*/
	if(s+2.6094379425048828125 >= 5.0*z) {
		goto S70;
	}
	/*
	Step 3
	*/
	t = log(z);
	if(s > t) {
		goto S70;
	}
	/*
	Step 4
	*/
	//    JJV added checker to see if log(alpha/(bb+w)) will 
	//    JJV overflow.  If so, we count the log as -INF, and
	//    JJV consequently evaluate conditional as true, i.e.
	//    JJV the algorithm rejects the trial and starts over
	//    JJV May not need this here since ALPHA > 2.0
	if(alpha/(bb+w) < tiny) {
		goto S40;
	}
	if(r+alpha*log(alpha/(bb+w)) < t) {
		goto S40;
	}
S70:
	/*
	Step 5
	*/
	if(a == aa) 
	{
		R = w/(bb+w);
		return R;
	}
	else
	{
		R = bb/(bb+w);
		return R;
	}
S100:
	/*
	Algorithm BC
	Initialize
	*/
	if(qsame) {
		goto S110;
	}
	aa = max(a,b);
	bb = min(a,b);
	alpha = aa+bb;
	beta = 1.0/bb;
	delta = 1.0+aa-bb;
	k1 = delta*(c1+c2*bb)/(aa*beta-c3);
	k2 = 0.25+(0.5+0.25/delta)*bb;
S110:
S120:
	u1 = genrand_randgenerate();
	/*
	Step 1
	*/
	u2 = genrand_randgenerate();
	if(u1 >= 0.5) {
		goto S130;
	}
	/*
	Step 2
	*/
	y = u1*u2;
	z = u1*y;
	if(0.25*u2+z-y >= k1) {
		goto S120;
	}
	goto S170;
S130:
	/*
	Step 3
	*/
	z = pow(u1,2.0)*u2;
	if(!(z <= 0.25)) {
		goto S160;
	}
	v = beta*log(u1/(1.0-u1));

	//  JJV instead of checking v > expmax at top, I will check
	//  JJV if aa < 1, then check the appropriate values

	if(aa > 1.0) {
		goto S135;
	}
	//  JJV A < 1 so it can help out if EXP(V) would overflow
	if(v > expmax) {
		goto S132;
	}
	w = aa*exp(v);
	goto S200;
S132:
	w = v + log(aa);
	if(w > expmax) {
		goto S140;
	}
	w = exp(w);
	goto S200;

	//  JJV in this case A > 1
S135:
	if(v > expmax) {
		goto S140;
	}
	w = exp(v);
	if(w > huge/aa) {
		goto S140;
	}
	w *= aa;
	goto S200;
S140:
	w = huge;
	goto S200;
S160:
	if(z >= k2) {
		goto S120;
	}
S170:
	/*
	Step 4
	Step 5
	*/
	v = beta*log(u1/(1.0-u1));
	//  JJV same kind of checking as above
	if(aa > 1.0) {
		goto S175;
	}
	//  JJV A < 1 so it can help out if EXP(V) would overflow
	if(v > expmax) {
		goto S172;
	}
	w = aa*exp(v);
	goto S190;
S172:
	w = v + log(aa);
	if(w > expmax) {
		goto S180;
	}
	w = exp(w);
	goto S190;
	//  JJV in this case A > 1
S175:
	if(v > expmax) {
		goto S180;
	}
	w = exp(v);
	if(w > huge/aa) {
		goto S180;
	}
	w *= aa;
	goto S190;

S180:
	w = huge;

	//  JJV here we also check to see if log overlows; if so, we treat it
	//  JJV as -INF, which means condition is true, i.e. restart
S190:
	if(alpha/(bb+w) < tiny) {
		goto S120;
	}
	if(alpha*(log(alpha/(bb+w))+v)-c4 < log(z)) {
		goto S120;
	}
S200:
	/*
	Step 6
	*/
	if(aa == a) 
	{
		R = w/(bb+w);
		return R;
	}
	else
	{
		R = bb/(bb+w);
		return R;
	}
}
