/*
 * Copyright (C) 2012 - Michael Baudin
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

#include <string.h>
#include <stdio.h>

#include "genrand.h"
#include "genrand_private.h"

// Initialize the RNG function to NULL.
double (* genrand_randfunction)(void) = NULL;

// Initialize the RNG function to NULL.
double (* genrand_randIntegerInRange)(double *a, double *b) = NULL;

// Initialize the message function to NULL.
void (* genrand_messagefunction)(char * message) = NULL;

// Configures the RNG function
void genrand_randSetFunction ( double (* f)(void) )
{
	genrand_randfunction = f;
}

// Configures the RNG function
void genrand_randIntegerInRangeSetFunction ( double (* f)(double *a, double *b) )
{
	genrand_randIntegerInRange = f;
}

// The function which generates random numbers uniform in [0,1).
double genrand_randgenerate()
{
	double R;
	double one=1;
	if (genrand_randfunction==NULL)
	{
		genrand_messageprint("genrand: No uniform random number generator configured.\n");
		genrand_messageprint("genrand: Please call genrand_randSetFunction at least once.\n");
		// This is NaN
		R=(one-one)/(one-one);
	}
	else
	{
		R = genrand_randfunction();
	}
	return R;
}

// The function which generates random integers uniform in [a,b].
double genrand_generateIntegerInRange(double *a , double *b)
{
	double R;
	double one=1;
	if (genrand_randfunction==NULL)
	{
		genrand_messageprint("genrand: No uniform random number generator configured.\n");
		genrand_messageprint("genrand: Please call genrand_randIntegerInRangeSetFunction at least once.\n");
		// This is NaN
		R=(one-one)/(one-one);
	}
	else
	{
		R = genrand_randIntegerInRange(a,b);
	}
	return R;
}

// Prints a message.
void genrand_messageprint ( char * message ) 
{
	if (genrand_messagefunction==NULL) {
		printf("%s\n",message);
	}
	else 
	{
		genrand_messagefunction(message);
	}
}

// Configures the message function
void genrand_messageSetFunction ( void (* f)(char * message) ) 
{
	genrand_messagefunction = f;
}

// THE MACHINE PRECISION
// dlamch_("p", (long int)1);
double genrand_doubleEps() 
{
	return 2.2204460492503131e-016;
}

// THE SMALLEST MAGNITUDE
// dlamch_("u", (long int)1);
double genrand_doubleTiny() 
{
	return 2.2250738585072014e-308;
}

// THE LARGEST MAGNITUDE
// dlamch_("o", (long int)1);
double genrand_doubleHuge() 
{
	return 1.7976931348623157e+308;
}

// Inf
double cdflib_doubleInfinite()
{
	double ret_val;
	double one;
	one=1.;
	ret_val = 1./(one-one);
	return ret_val;
}
